/********************************************************************
*      IMPLEMENTACAO DE UMA ULA/UC SEGUNDO A ARQUITETURA RISC       *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                               *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*      ARQUITETURA DE COMPUTADORES. Prof. Mrio Luis R. Oliveira    *
********************************************************************/

// wx -> necessaria para se utilizar wxWindows
#include "wx/wx.h"
#include "wx/image.h"
#include <string>
#include "risc.h"

// classe de aplicacao do wxWindows (como uma funcao int main())
class SimRiscApp : public wxApp
{
public:
    virtual bool OnInit();
};

// definicao da classe SimRiscPanel
class SimRiscPanel : public wxPanel
{
    public :
        // construtor
        SimRiscPanel(wxWindow *parent, int posX, int posY, int sizeX, int sizeY,
            long style = 0, const wxString& name = "SimRiscPanel");
        // destrutor
        ~SimRiscPanel();

        void OnPaint();
        void OnProcessarPassoAPasso();
        void OnProcesarTudo();
        void OnSobre();
        void OnCarregarInstrucoes();

    private :

        void MostrarStatus();

        Risc r;

        wxButton *processarPassoAPasso;
        wxButton *processarTudo;
        wxButton *sobre;
        wxButton *carregarInstrucoes;

        wxTextCtrl *memoria;
        wxTextCtrl *cache;
        wxTextCtrl *ciclosClock;

        wxBitmap  *logo;

        DECLARE_EVENT_TABLE()
};

// definicao da classe SimRiscFrame
class SimRiscFrame : public wxFrame
{
    public:
        // contrutor
        SimRiscFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

        // executada ao fechar o programa
        void OnExit(wxCommandEvent& event);

    private:
        // panel utilizado no programa
        SimRiscPanel* panel;

        DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(SimRiscFrame, wxFrame)
    EVT_MENU(wxID_EXIT, SimRiscFrame::OnExit)
END_EVENT_TABLE()

IMPLEMENT_APP(SimRiscApp)

// chamada ao iniciar o programa
bool SimRiscApp::OnInit()
{
#if wxUSE_LIBPNG
wxImage::AddHandler( new wxPNGHandler );
#endif
    // cria-se o frame
    SimRiscFrame *frame = new SimRiscFrame(NULL, "Sim Risc", wxDefaultPosition,
                            wxSize(695, 545));
    frame->Show(TRUE);

    return TRUE;
}

// contrutor do frame
SimRiscFrame::SimRiscFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                const wxSize& size, long style)
        : wxFrame(frame, -1, title, pos, size, style)
{
    // cria o panel
    panel = new SimRiscPanel(this, 0, 0, size.GetX(), size.GetY());

    SetBackgroundColour(*wxWHITE);
    Refresh();

    // cria uma barra de status
    CreateStatusBar(1);
    SetStatusText("Desenvolvido por: Anderson de Rezende Rocha e Jlio Csar Alves");
}

// executada ao fechar a janela
void SimRiscFrame::OnExit(wxCommandEvent& event)
{
    // desaloca o panel
    delete panel;
    Destroy();
}

enum { ID_PASSO_A_PASSO, ID_TUDO, ID_SOBRE, ID_CAR_INSTR, };

BEGIN_EVENT_TABLE(SimRiscPanel, wxPanel)
    EVT_BUTTON(ID_PASSO_A_PASSO, SimRiscPanel::OnProcessarPassoAPasso)
    EVT_BUTTON(ID_TUDO, SimRiscPanel::OnProcesarTudo)
    EVT_BUTTON(ID_SOBRE, SimRiscPanel::OnSobre)
    EVT_BUTTON(ID_CAR_INSTR, SimRiscPanel::OnCarregarInstrucoes)
    EVT_PAINT(SimRiscPanel::OnPaint)
END_EVENT_TABLE()

// construtor
SimRiscPanel::SimRiscPanel(wxWindow *parent, int posX, int posY,
    int sizeX, int sizeY, long style, const wxString& name):
    wxPanel(parent, posX, posY, sizeX, sizeY, style, name)
{
    processarPassoAPasso = new wxButton(this, ID_PASSO_A_PASSO, "Passo-a-Passo", wxPoint(415,85), wxSize(110, 30));
    carregarInstrucoes = new wxButton(this, ID_CAR_INSTR, "Carregar Arquivos", wxPoint(535,85), wxSize(110, 30));
    processarTudo = new wxButton(this, ID_TUDO, "Resultado Final", wxPoint(415,120), wxSize(110, 30));
    sobre = new wxButton(this, ID_SOBRE, "Sobre o SIMRISC", wxPoint(535,120), wxSize(110, 30));
    processarPassoAPasso->Enable(false);
    processarTudo->Enable(false);

    memoria = new wxTextCtrl(this, -1, "", wxPoint(485,195), wxSize(200,300), wxTE_MULTILINE | wxTE_READONLY );
    cache = new wxTextCtrl(this, -1, "cache", wxPoint(1280,230), wxSize(140,150), wxTE_MULTILINE | wxTE_READONLY );

//    (void) new wxStaticBox(this, -1, "Ciclo de Clock", wxPoint(300,170), wxSize(115, 45));
    ciclosClock = new wxTextCtrl(this, -1, "clock", wxPoint(930,185), wxSize(80,25), wxTE_READONLY );

    logo = (wxBitmap*) NULL;
    wxBitmap bitmap( 100, 100 );

    //wxImage image = bitmap.ConvertToImage();
    wxImage image(bitmap);

    wxMemoryDC dc;
    dc.SelectObject( bitmap );

#if wxUSE_LIBPNG

    if (!image.LoadFile("logo1.png"))
        wxMessageBox("Arquivo 'logo.png' no encontrado", "SimRisc", wxOK | wxICON_ERROR);
    else
        logo = new wxBitmap(image);
#endif // wxUSE_LIBPNG
    image.Destroy();

    //r.Set("memoria.txt", "instrucoes.txt");

    SetBackgroundColour(*wxWHITE);
    Refresh();
}

SimRiscPanel::~SimRiscPanel()
{
    delete processarPassoAPasso;
    delete processarTudo;
    delete sobre;
    delete carregarInstrucoes;
    delete memoria;
    delete cache;
    delete ciclosClock;

    delete logo;
}

void SimRiscPanel::OnPaint()
{
    wxPaintDC dc( this );
    PrepareDC( dc );

    wxString str("");

    dc.DrawRectangle(5, 5, 680, 160);
    
    if (logo->Ok())
        dc.DrawBitmap(*logo, 10, 10);
    dc.DrawText("SIMULADOR DE ARQUITETURA RISC", 400, 15);
    dc.DrawText("Opes Gerais do sistema", 450, 45); 

    dc.DrawText("Contador de Programa", 2, 170);
    dc.DrawRectangle(5, 190, 135, 25);
    dc.DrawText(r.GetRegistradores(0).c_str(), 8, 195);
    
    dc.DrawText( "Apontador", 180, 170);
    dc.DrawRectangle(145, 190, 135, 25);
    dc.DrawText(r.GetRegistradores(1).c_str(), 148, 195);
    
    dc.DrawText( "RPG 01", 50, 215);
    dc.DrawRectangle(5, 230, 135, 25);
    dc.DrawText(r.GetRegistradores(2).c_str(), 8, 235);
    
    dc.DrawText( "RPG 02", 185, 215);
    dc.DrawRectangle(145, 230, 135, 25);
    dc.DrawText(r.GetRegistradores(3).c_str(), 148, 235);
    
    dc.DrawText( "RPG 03", 50, 255);
    dc.DrawRectangle(5, 270, 135, 25);
    dc.DrawText(r.GetRegistradores(4).c_str(), 8, 275);
    
    dc.DrawText( "RPG 04", 185, 255);
    dc.DrawRectangle(145, 270, 135, 25);
    dc.DrawText(r.GetRegistradores(5).c_str(), 148, 275);
    
    dc.DrawText( "RPG 05", 50, 295);
    dc.DrawRectangle(5, 310, 135, 25);
    dc.DrawText(r.GetRegistradores(6).c_str(), 8, 315);
    
    dc.DrawText( "RPG 06", 185, 295);
    dc.DrawRectangle(145, 310, 135, 25);
    dc.DrawText(r.GetRegistradores(7).c_str(), 148, 315);
    
    dc.DrawText( "RPG 07", 50, 335);
    dc.DrawRectangle(5, 350, 135, 25);
    dc.DrawText(r.GetRegistradores(8).c_str(), 8, 355);
    
    dc.DrawText( "RPG 08", 185, 335);
    dc.DrawRectangle(145, 350, 135, 25);
    dc.DrawText(r.GetRegistradores(9).c_str(), 148, 355);
    
    dc.DrawText( "RPG 09", 50, 375);
    dc.DrawRectangle(5, 390, 135, 25);
    dc.DrawText(r.GetRegistradores(10).c_str(), 8, 395);
    
    dc.DrawText( "RPG 10", 185, 375);
    dc.DrawRectangle(145, 390, 135, 25);
    dc.DrawText(r.GetRegistradores(11).c_str(), 148, 395);

    dc.DrawText( "RPG 11", 50, 415);
    dc.DrawRectangle(5, 430, 135, 25);
    dc.DrawText(r.GetRegistradores(12).c_str(), 8, 435);
    
    dc.DrawText( "RPG 12", 185, 415);
    dc.DrawRectangle(145, 430, 135, 25);
    dc.DrawText(r.GetRegistradores(13).c_str(), 148, 435);    
    
    dc.DrawText( "RPG 13", 50, 455);
    dc.DrawRectangle(5, 470, 135, 25);
    dc.DrawText(r.GetRegistradores(14).c_str(), 8, 475);
    
    dc.DrawText( "RPG 14", 185, 455);
    dc.DrawRectangle(145, 470, 135, 25);
    dc.DrawText(r.GetRegistradores(15).c_str(), 148, 475); 
    
    str = "Ciclo de Clock: ";
    str << r.GetCiclosClock();
    dc.DrawText(str, 310, 195);


    dc.DrawRectangle(300, 260, 170, 190);
    dc.DrawText("CACHE", 360, 268);
    dc.DrawLine(300, 290, 470, 290);
        
    str = "OPCOD: ";
    str << r.GetCache(0);
    dc.DrawText(str, 310, 300);
    
    str = "OPERADOR 1: ";
    str << r.GetCache(1);
    dc.DrawText(str, 310, 330);
    
    str = "OPERADOR 2: ";
    str << r.GetCache(2);
    dc.DrawText(str, 310, 360);
    
    str = "OPERADOR 3: ";
    str << r.GetCache(3);
    dc.DrawText(str, 310, 390);
    
    str = "RESULTADO: ";
    str << r.GetCache(4);
    dc.DrawText(str, 310, 420);
    
    dc.DrawText("MEMRIA", 540, 170);
    memoria->Clear();
    for (int i=0; i < 256; i++)
    {
        if (r.GetMemoria(i) != "")
            (*memoria) << "[" << i << "]" << '\t' << r.GetMemoria(i).c_str() << "\n";
    }       
}

void SimRiscPanel::OnProcessarPassoAPasso()
{
    if (r.ProcessarPassoAPasso())
        //MostrarStatus();
        Refresh();
    else
        wxMessageBox("HALT atingido", "SimRisc", wxOK);
}

void SimRiscPanel::OnProcesarTudo()
{
    r.ProcessarTudo();
    //MostrarStatus();
    Refresh();
}

void SimRiscPanel::OnSobre()
{
    wxMessageBox("Simulador de Arquitetura Risc\n\nDesenvolvido por:\n"
                 "    Anderson de Rezende Rocha  undersun@comp.ufla.br\n"
                 "    Jlio Csar Alves  jcalves@comp.ufla.br\n\n"
                 "Em Linguagem C++, utilizando wxWindows\n\n"
                 "Discplina de Arquitetura de Computadores\n"
                 "Curso de Cincia da Computao\n"
                 "Universidade Federal de Lavras\n",
                 "SimRisc",wxOK);
}

void SimRiscPanel::OnCarregarInstrucoes()
{
    r.Limpar();
    wxFileDialog dialogo(this, "Carregar Instrues", "", "",
                                "*.txt", (wxOPEN | wxFILE_MUST_EXIST));
    if(dialogo.ShowModal() == wxID_OK)
    {
        if (!r.MapearInstrucoes(dialogo.GetPath().GetData()))
        {
            wxMessageBox("Arquivo de instrues contm alguma instruo no conhecida. Arquivo no carregado",
                        "SimRisc", wxOK | wxICON_ERROR);
        }
        else
        {
            wxFileDialog dialogo(this, "Carregar Memria", "", "",
                                        "*.txt", (wxOPEN | wxFILE_MUST_EXIST));
            if(dialogo.ShowModal() == wxID_OK)
            {
                if (!r.MapearMemoria(dialogo.GetPath().GetData()))
                    wxMessageBox("Arquivo de memria no pde ser carregado", "SimRisc", wxOK | wxICON_ERROR);
                else
                {
                    processarPassoAPasso->Enable(true);
                    processarTudo->Enable(true);
                }
            }
        }
    }
    Refresh();
}

void SimRiscPanel::MostrarStatus()
{
    ciclosClock->Clear();
    memoria->Clear();
    cache->Clear();

    (*ciclosClock) << r.GetCiclosClock();
    for (int i=0; i < 256; i++)
    {
        if (r.GetMemoria(i) != "")
            (*memoria) << "[" << i << "]" << '\t' << r.GetMemoria(i).c_str() << "\n";
    }

    for (int i=0; i < 5; i++)
        (*cache) << "[" << i << "]" << '\t' << r.GetCache(i) << "\n";
    Refresh();
}
